from dagster import EnvVar, asset, define_asset_job
from dagster._core.definitions.definitions_class import Definitions
from dagster._utils.env import environ
from dagster_sigma import (
    SigmaBaseUrl,
    SigmaOrganization,
    build_materialize_workbook_assets_definition,
    load_sigma_asset_specs,
)

fake_client_id = "fake_client_id"
fake_client_secret = "fake_client_secret"

with environ({"SIGMA_CLIENT_ID": fake_client_id, "SIGMA_CLIENT_SECRET": fake_client_secret}):
    fake_token = "fake_token"
    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=EnvVar("SIGMA_CLIENT_ID"),
        client_secret=EnvVar("SIGMA_CLIENT_SECRET"),
    )

    @asset
    def my_materializable_asset():
        pass

    sigma_specs = load_sigma_asset_specs(resource)
    sigma_assets = [
        build_materialize_workbook_assets_definition("sigma", spec)
        if spec.metadata.get("dagster_sigma/materialization_schedules")
        else spec
        for spec in sigma_specs
    ]

    defs = Definitions(
        assets=[my_materializable_asset, *sigma_assets],
        jobs=[define_asset_job("all_asset_job")],
        resources={"sigma": resource},
    )



import os
import uuid
from collections.abc import Callable, Iterator, Mapping
from contextlib import contextmanager, nullcontext
from typing import Any

import pytest
import responses
from dagster import AssetKey
from dagster._core.definitions.assets.definition.asset_spec import AssetSpec
from dagster._core.definitions.definitions_class import Definitions
from dagster._core.instance_for_test import instance_for_test
from dagster._utils.test.definitions import scoped_definitions_load_context
from dagster.components.testing import create_defs_folder_sandbox
from dagster_shared.utils import environ
from dagster_sigma import SigmaBaseUrl, SigmaComponent


@pytest.fixture(autouse=True)
def _setup(sigma_auth_token: str) -> Iterator:
    """Set up instance and activate responses mock for component tests."""
    responses.start()
    try:
        with instance_for_test():
            yield
    finally:
        responses.stop()
        responses.reset()


@contextmanager
def setup_sigma_component(
    defs_yaml_contents: dict[str, Any],
) -> Iterator[tuple[SigmaComponent, Definitions]]:
    """Sets up a components project with a sigma component based on provided params."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=SigmaComponent,
            defs_yaml_contents=defs_yaml_contents,
        )
        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (component, defs),
        ):
            assert isinstance(component, SigmaComponent)
            yield component, defs


def test_basic_component_load(sigma_sample_data: Any, sigma_auth_token: str) -> None:
    """Test basic component loading."""
    with (
        environ(
            {
                "SIGMA_BASE_URL": SigmaBaseUrl.AWS_US.value,
                "SIGMA_CLIENT_ID": uuid.uuid4().hex,
                "SIGMA_CLIENT_SECRET": uuid.uuid4().hex,
            }
        ),
        setup_sigma_component(
            defs_yaml_contents={
                "type": "dagster_sigma.SigmaComponent",
                "attributes": {
                    "organization": {
                        "base_url": "{{ env.SIGMA_BASE_URL }}",
                        "client_id": "{{ env.SIGMA_CLIENT_ID }}",
                        "client_secret": "{{ env.SIGMA_CLIENT_SECRET }}",
                    },
                },
            }
        ) as (
            component,
            defs,
        ),
    ):
        # Verify that we have the expected assets
        asset_keys = defs.resolve_asset_graph().get_all_asset_keys()
        assert AssetKey("Sample_Workbook") in asset_keys
        assert AssetKey("Orders_Dataset") in asset_keys
        assert component.organization.base_url == SigmaBaseUrl.AWS_US.value
        assert component.organization.client_id == os.environ["SIGMA_CLIENT_ID"]
        assert component.organization.client_secret == os.environ["SIGMA_CLIENT_SECRET"]


@pytest.mark.parametrize(
    "attributes, assertion, should_error",
    [
        ({"group_name": "group"}, lambda asset_spec: asset_spec.group_name == "group", False),
        (
            {"owners": ["team:analytics"]},
            lambda asset_spec: asset_spec.owners == ["team:analytics"],
            False,
        ),
        ({"tags": {"foo": "bar"}}, lambda asset_spec: asset_spec.tags.get("foo") == "bar", False),
        (
            {"kinds": ["snowflake", "dbt"]},
            lambda asset_spec: "snowflake" in asset_spec.kinds and "dbt" in asset_spec.kinds,
            False,
        ),
        (
            {"tags": {"foo": "bar"}, "kinds": ["snowflake", "dbt"]},
            lambda asset_spec: (
                "snowflake" in asset_spec.kinds
                and "dbt" in asset_spec.kinds
                and asset_spec.tags.get("foo") == "bar"
            ),
            False,
        ),
        ({"code_version": "1"}, lambda asset_spec: asset_spec.code_version == "1", False),
        (
            {"description": "some description"},
            lambda asset_spec: asset_spec.description == "some description",
            False,
        ),
        (
            {"metadata": {"foo": "bar"}},
            lambda asset_spec: asset_spec.metadata.get("foo") == "bar",
            False,
        ),
        (
            {"deps": ["customers"]},
            lambda asset_spec: (
                len(asset_spec.deps) == 1 and asset_spec.deps[0].asset_key == AssetKey("customers")
            ),
            False,
        ),
        (
            {"automation_condition": "{{ automation_condition.eager() }}"},
            lambda asset_spec: asset_spec.automation_condition is not None,
            False,
        ),
        (
            {"key_prefix": "cool_prefix"},
            lambda asset_spec: asset_spec.key.has_prefix(["cool_prefix"]),
            False,
        ),
    ],
    ids=[
        "group_name",
        "owners",
        "tags",
        "kinds",
        "tags-and-kinds",
        "code-version",
        "description",
        "metadata",
        "deps",
        "automation_condition",
        "key_prefix",
    ],
)
def test_translation(
    attributes: Mapping[str, Any],
    assertion: Callable[[AssetSpec], bool] | None,
    should_error: bool,
    sigma_sample_data: Any,
    sigma_auth_token: str,
) -> None:
    wrapper = pytest.raises(Exception) if should_error else nullcontext()
    with wrapper:
        body = {
            "type": "dagster_sigma.SigmaComponent",
            "attributes": {
                "organization": {
                    "base_url": SigmaBaseUrl.AWS_US.value,
                    "client_id": uuid.uuid4().hex,
                    "client_secret": uuid.uuid4().hex,
                },
            },
        }
        body["attributes"]["translation"] = attributes
        with (
            setup_sigma_component(
                defs_yaml_contents=body,
            ) as (_component, defs),
        ):
            if "key_prefix" in attributes:
                key = AssetKey(["cool_prefix", "Sample_Workbook"])
            elif "key" in attributes:
                # For key transformations, we need to determine the expected key
                key = next(iter(defs.resolve_asset_graph().get_all_asset_keys()))
            else:
                key = AssetKey("Sample_Workbook")

            assets_def = defs.get_assets_def(key)
            if assertion:
                assert assertion(assets_def.get_asset_spec(key))


def test_per_content_type_translation(sigma_sample_data: Any, sigma_auth_token: str) -> None:
    body = {
        "type": "dagster_sigma.SigmaComponent",
        "attributes": {
            "organization": {
                "base_url": SigmaBaseUrl.AWS_US.value,
                "client_id": uuid.uuid4().hex,
                "client_secret": uuid.uuid4().hex,
            },
            "translation": {
                "tags": {"custom_tag": "custom_value"},
                "for_workbook": {
                    "tags": {"is_workbook": "true"},
                },
                "for_dataset": {
                    "tags": {"is_dataset": "true"},
                },
            },
        },
    }
    with (
        setup_sigma_component(
            defs_yaml_contents=body,
        ) as (
            _component,
            defs,
        ),
    ):
        workbook_spec = defs.get_assets_def(AssetKey("Sample_Workbook")).get_asset_spec(
            AssetKey("Sample_Workbook")
        )
        assert workbook_spec.tags.get("custom_tag") == "custom_value"
        assert workbook_spec.tags.get("is_workbook") == "true"

        dataset_spec = defs.get_assets_def(AssetKey("Orders_Dataset")).get_asset_spec(
            AssetKey("Orders_Dataset")
        )
        assert dataset_spec.tags.get("custom_tag") == "custom_value"
        assert dataset_spec.tags.get("is_dataset") == "true"


class CustomSigmaComponent(SigmaComponent):
    def get_asset_spec(self, data) -> AssetSpec:
        # Override to add custom metadata and tags
        base_spec = super().get_asset_spec(data)
        return base_spec.replace_attributes(
            metadata={**base_spec.metadata, "custom_override": "test_value"},
            tags={**base_spec.tags, "custom_tag": "override_test"},
        )


def test_subclass_override_get_asset_spec(sigma_sample_data: Any, sigma_auth_token: str) -> None:
    """Test that subclasses of SigmaComponent can override get_asset_spec method."""
    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=CustomSigmaComponent,
            defs_yaml_contents={
                "type": "dagster_sigma_tests.test_components.CustomSigmaComponent",
                "attributes": {
                    "organization": {
                        "base_url": SigmaBaseUrl.AWS_US.value,
                        "client_id": uuid.uuid4().hex,
                        "client_secret": uuid.uuid4().hex,
                    },
                },
            },
        )
        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (_, defs),
        ):
            # Verify that the custom get_asset_spec method is being used
            assets_def = defs.get_assets_def(AssetKey("Sample_Workbook"))
            asset_spec = assets_def.get_asset_spec(AssetKey("Sample_Workbook"))

            # Check that our custom metadata and tags are present
            assert asset_spec.metadata["custom_override"] == "test_value"
            assert asset_spec.tags["custom_tag"] == "override_test"


@pytest.mark.parametrize(
    "defs_state_type",
    ["LOCAL_FILESYSTEM", "VERSIONED_STATE_STORAGE"],
)
def test_component_load_with_defs_state(
    sigma_sample_data: Any,
    sigma_auth_token: str,
    defs_state_type: str,
) -> None:
    import asyncio

    with create_defs_folder_sandbox() as sandbox:
        defs_path = sandbox.scaffold_component(
            component_cls=SigmaComponent,
            defs_yaml_contents={
                "type": "dagster_sigma.SigmaComponent",
                "attributes": {
                    "organization": {
                        "base_url": SigmaBaseUrl.AWS_US.value,
                        "client_id": uuid.uuid4().hex,
                        "client_secret": uuid.uuid4().hex,
                    },
                    "defs_state": {"management_type": defs_state_type},
                },
            },
        )
        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (component, defs),
        ):
            # First load, nothing there
            assert len(defs.resolve_asset_graph().get_all_asset_keys()) == 0
            assert isinstance(component, SigmaComponent)
            asyncio.run(component.refresh_state(sandbox.project_root))

        with (
            scoped_definitions_load_context(),
            sandbox.load_component_and_build_defs(defs_path=defs_path) as (component, defs),
        ):
            # Second load, should now have assets
            asset_keys = defs.resolve_asset_graph().get_all_asset_keys()
            assert AssetKey("Sample_Workbook") in asset_keys
            assert AssetKey("Orders_Dataset") in asset_keys



from pathlib import Path
from tempfile import TemporaryDirectory

import pytest
import responses
from click.testing import CliRunner
from dagster._core.code_pointer import CodePointer
from dagster._core.definitions import materialize
from dagster._core.definitions.asset_key import AssetKey
from dagster._core.definitions.reconstruct import (
    initialize_repository_def_from_pointer,
    reconstruct_repository_def_from_pointer,
)
from dagster._core.instance_for_test import instance_for_test
from dagster._utils.env import environ


@responses.activate
def test_snapshot_cli_rehydrate(sigma_auth_token: str, sigma_sample_data: None) -> None:
    with instance_for_test() as _instance, TemporaryDirectory() as temp_dir:
        from dagster_sigma.cli import sigma_snapshot_command

        temp_file = Path(temp_dir) / "snapshot.snap"
        out = CliRunner().invoke(
            sigma_snapshot_command,
            args=[
                "-f",
                str(Path(__file__).parent / "pending_repo_snapshot.py"),
                "--output-path",
                str(temp_file),
            ],
        )
        assert out.exit_code == 0

        calls = len(responses.calls)
        # Ensure that we can reconstruct the repository from the snapshot without making any calls
        with environ({"SIGMA_SNAPSHOT_PATH": str(temp_file)}):
            repository_def = initialize_repository_def_from_pointer(
                CodePointer.from_python_file(
                    str(Path(__file__).parent / "pending_repo_snapshot.py"), "defs", None
                ),
            )
            assert len(repository_def.assets_defs_by_key) == 2 + 1

            assert len(responses.calls) == calls


@responses.activate
def test_load_assets_organization_data(sigma_auth_token: str, sigma_sample_data: None) -> None:
    with instance_for_test() as _instance:
        # first, we resolve the repository to generate our cached metadata
        repository_def = initialize_repository_def_from_pointer(
            CodePointer.from_python_file(
                str(Path(__file__).parent / "pending_repo.py"), "defs", None
            ),
        )

        # 2 Sigma external assets, one materializable asset
        assert len(repository_def.assets_defs_by_key) == 2 + 1

        # No longer fetch owner for now, since this can be expensive
        # workbook_key = AssetKey("Sample_Workbook")
        # assert repository_def.assets_defs_by_key[workbook_key].owners_by_key[workbook_key] == [
        #     "ben@dagsterlabs.com"
        # ]

        calls = len(responses.calls)

        # Attempt to reconstruct - this should not call the expensive fetch functions
        # Responses will error if we do, since the mocked endpoints are only set up to be called once
        data = repository_def.repository_load_data

        # We use a separate file here just to ensure we get a fresh load
        recon_repository_def = reconstruct_repository_def_from_pointer(
            CodePointer.from_python_file(
                str(Path(__file__).parent / "pending_repo_2.py"), "defs", None
            ),
            data,
        )
        assert len(recon_repository_def.assets_defs_by_key) == 2 + 1

        assert len(responses.calls) == calls


@responses.activate
def test_materialize_workbook(
    sigma_auth_token: str, sigma_sample_data: None, sigma_materialization: None
) -> None:
    with instance_for_test() as _instance:
        # first, we resolve the repository to generate our cached metadata
        repository_def = initialize_repository_def_from_pointer(
            CodePointer.from_python_file(
                str(Path(__file__).parent / "materialize_workbook.py"), "defs", None
            ),
        )

        workbook_asset = repository_def.assets_defs_by_key[AssetKey(["Sample_Workbook"])]
        assert workbook_asset.is_materializable

        # materialize the workbook
        with environ({"SIGMA_CLIENT_ID": "fake", "SIGMA_CLIENT_SECRET": "fake"}):
            result = materialize([workbook_asset], raise_on_error=False)
            assert result.success


@responses.activate
def test_load_assets_organization_data_translator(
    sigma_auth_token: str, sigma_sample_data: None
) -> None:
    with instance_for_test() as _instance:
        repository_def = initialize_repository_def_from_pointer(
            CodePointer.from_python_file(
                str(Path(__file__).parent / "pending_repo_with_translator.py"), "defs", None
            ),
        )

        assert len(repository_def.assets_defs_by_key) == 2
        assert all(
            key.path[0] == "my_prefix" for key in repository_def.assets_defs_by_key.keys()
        ), repository_def.assets_defs_by_key


@responses.activate
def test_load_assets_organization_data_translator_legacy(
    sigma_auth_token: str, sigma_sample_data: None
) -> None:
    with instance_for_test() as _instance:
        with pytest.warns(
            DeprecationWarning,
            match=r"Support of `dagster_sigma_translator` as a Type\[DagsterSigmaTranslator\]",
        ):
            repository_def = initialize_repository_def_from_pointer(
                CodePointer.from_python_file(
                    str(Path(__file__).parent / "pending_repo_with_translator_legacy.py"),
                    "defs",
                    None,
                ),
            )

        assert len(repository_def.assets_defs_by_key) == 2
        assert all(
            key.path[0] == "my_prefix" for key in repository_def.assets_defs_by_key.keys()
        ), repository_def.assets_defs_by_key



import pytest
from dagster._core.definitions.asset_key import AssetKey
from dagster._core.definitions.assets.definition.asset_spec import AssetSpec
from dagster._core.definitions.metadata.table import TableColumn, TableSchema
from dagster._core.definitions.tags import build_kind_tag_key
from dagster_sigma.translator import (
    DagsterSigmaTranslator,
    SigmaDataset,
    SigmaDatasetTranslatorData,
    SigmaOrganizationData,
    SigmaTable,
    SigmaWorkbook,
    SigmaWorkbookTranslatorData,
)

from dagster_sigma_tests.conftest import (
    SAMPLE_DATASET_DATA,
    SAMPLE_DATASET_INODE,
    SAMPLE_TABLE_DATA,
    SAMPLE_TABLE_INODE,
    SAMPLE_WORKBOOK_DATA,
)


def test_workbook_translation() -> None:
    sample_workbook = SigmaWorkbook(
        properties=SAMPLE_WORKBOOK_DATA,
        datasets={SAMPLE_DATASET_INODE},
        owner_email="ben@dagsterlabs.com",
        direct_table_deps={SAMPLE_TABLE_INODE},
        lineage=[],
        materialization_schedules=None,
    )

    sample_dataset = SigmaDataset(properties=SAMPLE_DATASET_DATA, columns=set(), inputs=set())

    translator = DagsterSigmaTranslator()

    asset_spec = translator.get_asset_spec(
        SigmaWorkbookTranslatorData(
            workbook=sample_workbook,
            organization_data=SigmaOrganizationData(
                workbooks=[sample_workbook],
                datasets=[sample_dataset],
                tables=[SigmaTable(properties=SAMPLE_TABLE_DATA)],
            ),
        )
    )

    assert asset_spec.key.path == ["Sample_Workbook"]
    assert asset_spec.metadata["dagster_sigma/web_url"].value == SAMPLE_WORKBOOK_DATA["url"]
    assert asset_spec.metadata["dagster_sigma/version"] == 5
    assert asset_spec.metadata["dagster_sigma/created_at"].value == 1726176169.072
    assert build_kind_tag_key("sigma") in asset_spec.tags
    assert build_kind_tag_key("workbook") in asset_spec.tags
    assert asset_spec.owners == ["ben@dagsterlabs.com"]
    assert {dep.asset_key for dep in asset_spec.deps} == {
        AssetKey(["Orders_Dataset"]),
        AssetKey(["my_database", "my_schema", "payments"]),
    }


def test_dataset_translation() -> None:
    sample_dataset = SigmaDataset(
        properties=SAMPLE_DATASET_DATA,
        columns={"user_id", "date", "order_id"},
        inputs={"TESTDB.JAFFLE_SHOP.STG_ORDERS"},
    )

    translator = DagsterSigmaTranslator()

    asset_spec = translator.get_asset_spec(
        SigmaDatasetTranslatorData(
            dataset=sample_dataset,
            organization_data=SigmaOrganizationData(
                workbooks=[], datasets=[sample_dataset], tables=[]
            ),
        )
    )

    assert asset_spec.key.path == ["Orders_Dataset"]
    assert asset_spec.metadata["dagster_sigma/web_url"].value == SAMPLE_DATASET_DATA["url"]
    assert asset_spec.metadata["dagster_sigma/created_at"].value == 1726175777.83
    assert asset_spec.metadata["dagster/column_schema"] == TableSchema(
        columns=[
            TableColumn(name="date"),
            TableColumn(name="order_id"),
            TableColumn(name="user_id"),
        ]
    )

    assert asset_spec.description == "Wow, cool orders dataset"

    assert build_kind_tag_key("sigma") in asset_spec.tags
    assert build_kind_tag_key("dataset") in asset_spec.tags
    assert {dep.asset_key for dep in asset_spec.deps} == {
        AssetKey(["testdb", "jaffle_shop", "stg_orders"])
    }


def test_dataset_translation_custom_translator() -> None:
    class MyCustomTranslator(DagsterSigmaTranslator):
        def get_asset_spec(
            self, data: SigmaDatasetTranslatorData | SigmaWorkbookTranslatorData
        ) -> AssetSpec:
            spec = super().get_asset_spec(data)
            if isinstance(data, SigmaDatasetTranslatorData):
                spec = spec.replace_attributes(
                    key=spec.key.with_prefix("sigma"), description="Custom description"
                )
            return spec

    sample_dataset = SigmaDataset(
        properties=SAMPLE_DATASET_DATA,
        columns={"user_id", "date", "order_id"},
        inputs={"TESTDB.JAFFLE_SHOP.STG_ORDERS"},
    )

    translator = MyCustomTranslator()

    asset_spec = translator.get_asset_spec(
        SigmaDatasetTranslatorData(
            dataset=sample_dataset,
            organization_data=SigmaOrganizationData(
                workbooks=[], datasets=[sample_dataset], tables=[]
            ),
        )
    )

    assert asset_spec.key.path == ["sigma", "Orders_Dataset"]
    assert asset_spec.description == "Custom description"


def test_workbook_translation_with_missing_table(caplog: pytest.LogCaptureFixture) -> None:
    """Test that workbook translation handles missing table inodes gracefully.

    This can happen when warn_on_lineage_fetch_error is True and some table
    metadata failed to fetch but table dependencies were still recorded.
    """
    # Create a workbook that references a table inode that doesn't exist
    missing_table_inode = "inode-MissingTableXYZ123"
    sample_workbook = SigmaWorkbook(
        properties=SAMPLE_WORKBOOK_DATA,
        datasets={SAMPLE_DATASET_INODE},
        owner_email="ben@dagsterlabs.com",
        direct_table_deps={SAMPLE_TABLE_INODE, missing_table_inode},  # One exists, one doesn't
        lineage=[],
        materialization_schedules=None,
    )

    sample_dataset = SigmaDataset(properties=SAMPLE_DATASET_DATA, columns=set(), inputs=set())

    translator = DagsterSigmaTranslator()

    # Organization data only has SAMPLE_TABLE_DATA, not the missing table
    asset_spec = translator.get_asset_spec(
        SigmaWorkbookTranslatorData(
            workbook=sample_workbook,
            organization_data=SigmaOrganizationData(
                workbooks=[sample_workbook],
                datasets=[sample_dataset],
                tables=[SigmaTable(properties=SAMPLE_TABLE_DATA)],  # Only one table
            ),
        )
    )

    # Should still generate a valid asset spec
    assert asset_spec.key.path == ["Sample_Workbook"]

    # Should include the dataset dependency
    dep_keys = {dep.asset_key for dep in asset_spec.deps}
    assert AssetKey(["Orders_Dataset"]) in dep_keys

    # Should include the table that was found
    assert AssetKey(["my_database", "my_schema", "payments"]) in dep_keys

    # Should only have these two dependencies (missing table should be skipped)
    assert len(dep_keys) == 2

    # Should have logged a warning about the missing table
    assert any(
        "not found in organization data" in record.message and missing_table_inode in record.message
        for record in caplog.records
    )



import asyncio
import json
import uuid
from unittest import mock

import pytest
import responses
from aiohttp import ClientResponseError, hdrs
from aioresponses import aioresponses
from dagster_sigma import SigmaBaseUrl, SigmaOrganization
from dagster_sigma.resource import SigmaFilter, _inode_from_url

from dagster_sigma_tests.utils import get_requests


@responses.activate
def test_authorization(sigma_auth_token: str) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    assert resource.api_token == sigma_auth_token

    assert len(responses.calls) == 1
    assert responses.calls[0].request.headers["Content-Type"] == "application/x-www-form-urlencoded"
    assert responses.calls[0].request.body == "&".join(
        [
            f"{k}={v}"
            for k, v in {
                "grant_type": "client_credentials",
                "client_id": fake_client_id,
                "client_secret": fake_client_secret,
            }.items()
        ]
    )


@responses.activate
def test_basic_fetch(sigma_auth_token: str, responses: aioresponses) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    responses.add(
        method=hdrs.METH_GET,
        url=f"{SigmaBaseUrl.AWS_US.value}/v2/datasets",
        body=json.dumps({}),
        status=200,
    )

    asyncio.run(resource._fetch_json_async("datasets"))  # noqa: SLF001

    calls = get_requests(responses)
    assert len(calls) == 1
    assert calls[0].headers["Authorization"] == f"Bearer {sigma_auth_token}"


@responses.activate
def test_model_organization_data(sigma_auth_token: str, sigma_sample_data: None) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    data = asyncio.run(
        resource.build_organization_data(
            sigma_filter=None, fetch_column_data=True, fetch_lineage_data=True
        )
    )

    assert len(data.workbooks) == 1
    assert data.workbooks[0].properties["name"] == "Sample Workbook"

    assert len(data.datasets) == 1
    assert data.workbooks[0].datasets == {_inode_from_url(data.datasets[0].properties["url"])}

    assert data.datasets[0].properties["name"] == "Orders Dataset"
    assert data.datasets[0].columns == {
        "Customer Id",
        "Order Date",
        "Order Id",
        "Modified Date",
        "Status",
    }
    assert data.datasets[0].inputs == {"TESTDB.JAFFLE_SHOP.STG_ORDERS"}


@pytest.mark.parametrize("filter_type", ["workbook_folders", "workbook_names", "both"])
@responses.activate
def test_model_organization_data_filter(
    sigma_auth_token: str, sigma_sample_data: None, filter_type: str
) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    non_matching_filter = (
        {"workbook_folders": [("My Documents", "Test Folder")]}
        if filter_type == "workbook_folders"
        else {"workbooks": [("My Documents", "Test Folder", "Sample Workbook")]}
    )
    matching_filter = (
        {"workbook_folders": [("My Documents", "My Subfolder")]}
        if filter_type == "workbook_folders"
        else {"workbooks": [("My Documents", "My Subfolder", "Sample Workbook")]}
    )

    with mock.patch.object(
        SigmaOrganization,
        "_fetch_pages_for_workbook",
        wraps=resource._fetch_pages_for_workbook,  # noqa: SLF001
    ) as mock_fetch_pages:
        data = asyncio.run(
            resource.build_organization_data(
                sigma_filter=SigmaFilter(**non_matching_filter, include_unused_datasets=True),
                fetch_column_data=True,
                fetch_lineage_data=True,
            )
        )
        assert len(data.workbooks) == 0
        assert len(data.datasets) == 1

        # We don't fetch the workbooks, so we shouldn't have made any calls
        assert len(mock_fetch_pages.mock_calls) == 0

        data = asyncio.run(
            resource.build_organization_data(
                sigma_filter=SigmaFilter(**non_matching_filter, include_unused_datasets=False),
                fetch_column_data=True,
                fetch_lineage_data=True,
            )
        )
        assert len(data.workbooks) == 0
        assert len(data.datasets) == 0

        # We still don't fetch the workbooks, so we shouldn't have made any calls
        assert len(mock_fetch_pages.mock_calls) == 0

        data = asyncio.run(
            resource.build_organization_data(
                sigma_filter=SigmaFilter(**matching_filter, include_unused_datasets=False),
                fetch_column_data=True,
                fetch_lineage_data=True,
            )
        )

        assert len(data.workbooks) == 1
        assert len(data.datasets) == 1
        assert data.workbooks[0].properties["name"] == "Sample Workbook"

        # We fetch the workbook thrice, once for generating the workbook object,
        # once for fetching column data, and once for fetching lineage data
        # (the results are cached, so we don't actually make three calls out to the API)
        assert len(mock_fetch_pages.mock_calls) == 3

        data = asyncio.run(
            resource.build_organization_data(
                sigma_filter=SigmaFilter(
                    workbook_folders=[("My Documents",)], workbooks=[("Does", "Not", "Exist")]
                ),
                fetch_column_data=True,
                fetch_lineage_data=True,
            )
        )

        assert len(data.workbooks) == 1
        assert data.workbooks[0].properties["name"] == "Sample Workbook"


@responses.activate
def test_model_organization_data_skip_fetch_col_data(
    sigma_auth_token: str, sigma_sample_data: None
) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    data = asyncio.run(
        resource.build_organization_data(
            sigma_filter=None, fetch_column_data=False, fetch_lineage_data=True
        )
    )

    assert len(data.datasets) == 1
    assert data.workbooks[0].datasets == {_inode_from_url(data.datasets[0].properties["url"])}

    assert data.datasets[0].properties["name"] == "Orders Dataset"
    assert data.datasets[0].columns == set()


@responses.activate
def test_model_organization_data_skip_fetch_lineage(
    sigma_auth_token: str, sigma_sample_data: None
) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    data = asyncio.run(
        resource.build_organization_data(
            sigma_filter=None, fetch_column_data=True, fetch_lineage_data=False
        )
    )

    assert len(data.datasets) == 1
    assert data.workbooks[0].datasets == set()


@responses.activate
def test_model_organization_data_warn_err(
    sigma_auth_token: str, sigma_sample_data: None, lineage_warn: None
) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
        warn_on_lineage_fetch_error=False,
    )

    with pytest.raises(ClientResponseError):
        asyncio.run(
            resource.build_organization_data(
                sigma_filter=None, fetch_column_data=True, fetch_lineage_data=True
            )
        )


@responses.activate
def test_model_organization_data_warn_no_err(
    sigma_auth_token: str, sigma_sample_data: None, lineage_warn: None
) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource_warn = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
        warn_on_lineage_fetch_error=True,
    )

    data = asyncio.run(
        resource_warn.build_organization_data(
            sigma_filter=None, fetch_column_data=True, fetch_lineage_data=True
        )
    )

    assert len(data.workbooks) == 1
    assert data.workbooks[0].properties["name"] == "Sample Workbook"

    assert len(data.datasets) == 1
    assert data.workbooks[0].datasets == {_inode_from_url(data.datasets[0].properties["url"])}


@responses.activate
def test_rate_limited_recover(sigma_auth_token: str, sigma_sample_rate_limited_data: None) -> None:
    fake_client_id = uuid.uuid4().hex
    fake_client_secret = uuid.uuid4().hex

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=fake_client_id,
        client_secret=fake_client_secret,
    )

    data = asyncio.run(
        resource.build_organization_data(
            sigma_filter=None, fetch_column_data=True, fetch_lineage_data=True
        )
    )

    assert len(data.datasets) == 1
    assert data.workbooks[0].datasets == {_inode_from_url(data.datasets[0].properties["url"])}

    assert data.datasets[0].properties["name"] == "Orders Dataset"



import os

from dagster import EnvVar, asset, define_asset_job
from dagster._core.definitions.definitions_class import Definitions
from dagster._utils.env import environ
from dagster_sigma import SigmaBaseUrl, SigmaOrganization, load_sigma_asset_specs

fake_client_id = "fake_client_id"
fake_client_secret = "fake_client_secret"
snapshot_path = os.getenv("SIGMA_SNAPSHOT_PATH") or None

with environ({"SIGMA_CLIENT_ID": fake_client_id, "SIGMA_CLIENT_SECRET": fake_client_secret}):
    fake_token = "fake_token"
    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=EnvVar("SIGMA_CLIENT_ID"),
        client_secret=EnvVar("SIGMA_CLIENT_SECRET"),
    )

    @asset
    def my_materializable_asset():
        pass

    sigma_specs = load_sigma_asset_specs(resource, snapshot_path=snapshot_path)
    defs = Definitions(
        assets=[my_materializable_asset, *sigma_specs], jobs=[define_asset_job("all_asset_job")]
    )



from typing import Any

from aioresponses import aioresponses


class RequestToMock:
    def __init__(self, value: dict[str, Any]) -> None:
        self.value = value

    @property
    def headers(self) -> dict[str, Any]:
        return self.value["headers"]

    @property
    def body(self) -> str:
        return self.value["data"]

    def urlencoded_form_data(self) -> str:
        return self.value["data"]().decode()


def get_requests(responses: aioresponses) -> list[RequestToMock]:
    return [RequestToMock(value[0].kwargs) for value in responses.requests.values()]



import json
import uuid
from collections.abc import Iterator
from typing import Any

import pytest
import responses as request_responses
from aiohttp import hdrs
from aioresponses import aioresponses
from dagster_sigma import SigmaBaseUrl


@pytest.fixture(name="responses")
def mock_responses() -> Iterator[aioresponses]:
    with aioresponses() as m:
        yield m


@pytest.fixture(name="sigma_auth_token")
def sigma_auth_fixture() -> str:
    fake_access_token: str = uuid.uuid4().hex

    request_responses.add(
        method=request_responses.POST,
        url=f"{SigmaBaseUrl.AWS_US.value}/v2/auth/token",
        json={"access_token": fake_access_token},
        status=200,
    )

    return fake_access_token


def _build_paginated_response(
    items: list[dict[str, Any]], slice_start: int | None = None, slice_end: int | None = None
) -> dict[str, Any]:
    has_more = False
    next_page = None
    items_to_return = items

    if slice_start is not None or slice_end is not None:
        items_to_return = items[slice_start:slice_end]
    if slice_end is not None and slice_end < len(items):
        has_more = True
        next_page = slice_end

    return {
        "entries": items_to_return,
        "hasMore": has_more,
        "total": len(items),
        "nextPage": next_page,
    }


SAMPLE_WORKBOOK_DATA = {
    "workbookId": "4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e",
    "workbookUrlId": "2opi6VLEne4BaPyj00US50",
    "createdBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
    "updatedBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
    "createdAt": "2024-09-12T21:22:49.072Z",
    "updatedAt": "2024-09-12T22:20:39.848Z",
    "name": "Sample Workbook",
    "url": "https://app.sigmacomputing.com/dagster-labs/workbook/2opi6VLEne4BaPyj00US50",
    "path": "My Documents/My Subfolder",
    "latestVersion": 5,
    "ownerId": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
}

SAMPLE_DATASET_INODE = "inode-Iq557kfHN8KRu76HdGSWi"
SAMPLE_DATASET_DATA = {
    "datasetId": "178a6bb5-c0e7-4bef-a739-f12710492f16",
    "createdBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
    "updatedBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
    "createdAt": "2024-09-12T21:16:17.830Z",
    "updatedAt": "2024-09-12T21:19:31.000Z",
    "name": "Orders Dataset",
    "description": "Wow, cool orders dataset",
    "url": "https://app.sigmacomputing.com/dagster-labs/b/Iq557kfHN8KRu76HdGSWi",
}

SAMPLE_TABLE_INODE = "inode-C5ZiVq1GkaANMaD334AnA"
SAMPLE_TABLE_DATA = {
    "path": "Content Root/My Database/My Schema",
    "urlId": SAMPLE_TABLE_INODE.split("-")[-1],
    "name": "Payments",
}


@pytest.fixture(name="lineage_warn")
def lineage_warn_fixture(responses: aioresponses) -> None:
    # clear out the existing response
    url_to_warn = "https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/lineage/elements/_MuHPbskp0"
    key_to_del = None
    for key, response in responses._matches.items():  # noqa: SLF001
        if str(response.url_or_pattern) == url_to_warn:
            key_to_del = key

    assert key_to_del
    del responses._matches[key_to_del]  # noqa: SLF001

    # failed responses don't get cached
    for _ in range(2):
        responses.add(
            method=hdrs.METH_GET,
            url=url_to_warn,
            status=400,
        )


@pytest.fixture(name="sigma_materialization")
def sigma_materialization_fixture(responses: aioresponses) -> None:
    # Trigger materialization, check status, check status again
    request_responses.add(
        method=request_responses.POST,
        url=f"{SigmaBaseUrl.AWS_US.value}/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/materializations",
        status=200,
        body=json.dumps({"materializationId": "foobar"}),
    )
    request_responses.add(
        method=request_responses.GET,
        url=f"{SigmaBaseUrl.AWS_US.value}/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/materializations/foobar",
        status=200,
        body=json.dumps({"materializationId": "foobar", "status": "pending"}),
    )
    request_responses.add(
        method=request_responses.GET,
        url=f"{SigmaBaseUrl.AWS_US.value}/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/materializations/foobar",
        status=200,
        body=json.dumps({"materializationId": "foobar", "status": "ready"}),
    )


def _add_sample_responses(responses: aioresponses) -> None:
    # Single workbook, dataset
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks?limit=1000",
        body=json.dumps(_build_paginated_response([SAMPLE_WORKBOOK_DATA])),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/datasets?limit=1000",
        body=json.dumps(_build_paginated_response([SAMPLE_DATASET_DATA])),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/files?limit=1000&typeFilters=table",
        body=json.dumps(_build_paginated_response([SAMPLE_TABLE_DATA])),
        status=200,
    )

    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/pages?limit=1000",
        body=json.dumps(_build_paginated_response([{"pageId": "qwMyyHBCuC", "name": "Page 1"}])),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/materialization-schedules?limit=1000",
        body=json.dumps(_build_paginated_response([{"sheetId": "qwMyyHBCuC"}])),
        status=200,
    )
    elements = [
        {
            "elementId": "_MuHPbskp0",
            "type": "table",
            "name": "sample elementl",
            "columns": [
                "Order Id",
                "Customer Id",
                "workbook renamed date",
                "Modified Date",
            ],
            "vizualizationType": "levelTable",
        },
        {
            "elementId": "V29pknzHb6",
            "type": "visualization",
            "name": "Count of Order Date by Status",
            "columns": [
                "Order Id",
                "Customer Id",
                "Order Date",
                "Status",
                "Modified Date",
                "Count of Order Date",
            ],
            "vizualizationType": "bar",
        },
        {
            "elementId": "A49pknzHb1",
            "type": "table",
            "name": "Payments",
            "columns": [],
            "vizualizationType": "levelTable",
        },
    ]
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/pages/qwMyyHBCuC/elements?limit=1000",
        body=json.dumps(_build_paginated_response(elements, 0, 1)),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/pages/qwMyyHBCuC/elements?limit=1000&page=1",
        body=json.dumps(_build_paginated_response(elements, 1, 3)),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/lineage/elements/_MuHPbskp0",
        body=json.dumps(
            {
                "dependencies": {
                    "qy_ARjTKcT": {
                        "nodeId": "qy_ARjTKcT",
                        "type": "sheet",
                        "name": "sample elementl",
                        "elementId": "_MuHPbskp0",
                    },
                    "inode-Iq557kfHN8KRu76HdGSWi": {
                        "nodeId": "inode-Iq557kfHN8KRu76HdGSWi",
                        "type": "dataset",
                        "name": "Orders Dataset",
                    },
                },
                "edges": [
                    {
                        "source": "inode-Iq557kfHN8KRu76HdGSWi",
                        "target": "qy_ARjTKcT",
                        "type": "source",
                    }
                ],
            }
        ),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/elements/_MuHPbskp0/columns",
        body=json.dumps(
            _build_paginated_response(
                [
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Order Id", "label": "Order Id"},
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Customer Id", "label": "Customer Id"},
                    {
                        "columnId": "inode-Iq557kfHN8KRu76HdGSWi/Order Date",
                        "label": "workbook renamed date",
                    },
                    {
                        "columnId": "inode-Iq557kfHN8KRu76HdGSWi/Modified Date",
                        "label": "Modified Date",
                    },
                ]
            )
        ),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/lineage/elements/A49pknzHb1",
        body=json.dumps(
            {
                "dependencies": {
                    SAMPLE_TABLE_INODE: {
                        "nodeId": SAMPLE_TABLE_INODE,
                        "type": "table",
                        "name": "Payments",
                    },
                },
            }
        ),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/elements/A49pknzHb1/columns",
        body=json.dumps(_build_paginated_response([])),
        status=200,
    )

    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/lineage/elements/V29pknzHb6",
        body=json.dumps(
            {
                "dependencies": {
                    "SBvQYKT6ui": {
                        "nodeId": "SBvQYKT6ui",
                        "type": "sheet",
                        "name": "Count of Order Date by Status",
                        "elementId": "V29pknzHb6",
                    },
                    "inode-Iq557kfHN8KRu76HdGSWi": {
                        "nodeId": "inode-Iq557kfHN8KRu76HdGSWi",
                        "type": "dataset",
                        "name": "Orders Dataset",
                    },
                },
                "edges": [
                    {
                        "source": "inode-Iq557kfHN8KRu76HdGSWi",
                        "target": "SBvQYKT6ui",
                        "type": "source",
                    }
                ],
            }
        ),
        status=200,
    )

    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/elements/V29pknzHb6/columns",
        body=json.dumps(
            _build_paginated_response(
                [
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Order Id", "label": "Order Id"},
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Customer Id", "label": "Customer Id"},
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Order Date", "label": "Order Date"},
                    {"columnId": "inode-Iq557kfHN8KRu76HdGSWi/Status", "label": "Status"},
                    {
                        "columnId": "inode-Iq557kfHN8KRu76HdGSWi/Modified Date",
                        "label": "Modified Date",
                    },
                    {"columnId": "VBKAHQgx58", "label": "Count of Order Date"},
                ]
            )
        ),
        status=200,
    )
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks/4ea60fe9-f487-43b0-aa7a-3ef43ca3a90e/queries",
        body=json.dumps(
            _build_paginated_response(
                [
                    {
                        "elementId": "_MuHPbskp0",
                        "name": "sample elementl",
                        "sql": 'select ORDER_ID_7 "Order Id", CUSTOMER_ID_8 "Customer Id", CAST_DATE_TO_DATETIME_9 "workbook renamed date", CAST_DATE_TO_DATETIME_11 "Modified Date" from (select ORDER_ID ORDER_ID_7, CUSTOMER_ID CUSTOMER_ID_8, ORDER_DATE::timestamp_ltz CAST_DATE_TO_DATETIME_9, ORDER_DATE::timestamp_ltz CAST_DATE_TO_DATETIME_11 from (select * from TESTDB.JAFFLE_SHOP.STG_ORDERS STG_ORDERS limit 1000) Q1) Q2 limit 1000\n\n-- Sigma \u03a3 {"request-id":"69ac9a35-64b3-4840-a53d-3aed43a575ec","email":"ben@dagsterlabs.com"}',
                    },
                    {
                        "elementId": "V29pknzHb6",
                        "name": "Count of Order Date by Status",
                        "sql": 'with Q1 as (select ORDER_ID, CUSTOMER_ID, STATUS, ORDER_DATE::timestamp_ltz CAST_DATE_TO_DATETIME_5, ORDER_DATE::timestamp_ltz CAST_DATE_TO_DATETIME_6 from TESTDB.JAFFLE_SHOP.STG_ORDERS STG_ORDERS) select STATUS_10 "Status", COUNT_22 "Count of Order Date", ORDER_ID_7 "Order Id", CUSTOMER_ID_8 "Customer Id", CAST_DATE_TO_DATETIME_7 "Order Date", CAST_DATE_TO_DATETIME_8 "Modified Date" from (select Q3.ORDER_ID_7 ORDER_ID_7, Q3.CUSTOMER_ID_8 CUSTOMER_ID_8, Q3.CAST_DATE_TO_DATETIME_7 CAST_DATE_TO_DATETIME_7, Q3.STATUS_10 STATUS_10, Q3.CAST_DATE_TO_DATETIME_8 CAST_DATE_TO_DATETIME_8, Q6.COUNT_22 COUNT_22, Q6.STATUS_11 STATUS_11 from (select ORDER_ID ORDER_ID_7, CUSTOMER_ID CUSTOMER_ID_8, CAST_DATE_TO_DATETIME_6 CAST_DATE_TO_DATETIME_7, STATUS STATUS_10, CAST_DATE_TO_DATETIME_5 CAST_DATE_TO_DATETIME_8 from Q1 Q2 order by STATUS_10 asc limit 1000) Q3 left join (select count(CAST_DATE_TO_DATETIME_7) COUNT_22, STATUS_10 STATUS_11 from (select CAST_DATE_TO_DATETIME_6 CAST_DATE_TO_DATETIME_7, STATUS STATUS_10 from Q1 Q4) Q5 group by STATUS_10) Q6 on equal_null(Q3.STATUS_10, Q6.STATUS_11)) Q8 order by STATUS_10 asc limit 1000\n\n-- Sigma \u03a3 {"request-id":"69ac9a35-64b3-4840-a53d-3aed43a575ec","email":"ben@dagsterlabs.com"}',
                    },
                ]
            )
        ),
        status=200,
    )

    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/members",
        body=json.dumps(
            _build_paginated_response(
                [
                    {
                        "organizationId": "4d55c33f-dbb8-4426-9d70-97742e01f002",
                        "memberId": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
                        "memberType": "admin",
                        "firstName": "Ben",
                        "lastName": "Pankow",
                        "email": "ben@dagsterlabs.com",
                        "profileImgUrl": None,
                        "createdBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
                        "updatedBy": "8TUQL5YbOwebkUGS0SAdqxlU5R0gD",
                        "createdAt": "2024-09-12T20:44:19.736Z",
                        "updatedAt": "2024-09-12T20:44:19.736Z",
                        "homeFolderId": "bd7e1b16-dad3-45d4-91e7-5687be2819cc",
                        "userKind": "internal",
                    },
                    {
                        "organizationId": "4d55c33f-dbb8-4426-9d70-97742e01f002",
                        "memberId": "SigmaSchedulerRobot",
                        "memberType": "admin",
                        "firstName": "Scheduler",
                        "lastName": "User",
                        "email": "scheduler-robot@sigmacomputing.com",
                        "profileImgUrl": None,
                        "createdBy": "SigmaSchedulerRobot",
                        "updatedBy": "SigmaSchedulerRobot",
                        "createdAt": "2024-09-12T20:44:20.182Z",
                        "updatedAt": "2024-09-12T20:44:20.182Z",
                        "homeFolderId": "4537ec2e-ced7-4aa6-b511-ed0437e0165f",
                        "userKind": "internal",
                    },
                ]
            )
        ),
        status=200,
    )


@pytest.fixture(name="sigma_sample_data")
def sigma_sample_data_fixture(responses: aioresponses) -> None:
    _add_sample_responses(responses)


@pytest.fixture(name="sigma_sample_rate_limited_data")
def sigma_sample_rate_limited_data_fixture(responses: aioresponses) -> None:
    # rate limited, but recovers
    responses.add(
        method=hdrs.METH_GET,
        url="https://aws-api.sigmacomputing.com/v2/workbooks?limit=1000",
        status=429,
    )
    _add_sample_responses(responses)



from dagster import EnvVar, define_asset_job
from dagster._core.definitions.assets.definition.asset_spec import AssetSpec
from dagster._core.definitions.definitions_class import Definitions
from dagster._utils.env import environ
from dagster_sigma import (
    DagsterSigmaTranslator,
    SigmaBaseUrl,
    SigmaOrganization,
    load_sigma_asset_specs,
)

fake_client_id = "fake_client_id"
fake_client_secret = "fake_client_secret"

with environ({"SIGMA_CLIENT_ID": fake_client_id, "SIGMA_CLIENT_SECRET": fake_client_secret}):
    fake_token = "fake_token"

    class MyCoolTranslator(DagsterSigmaTranslator):
        def get_asset_spec(self, data) -> AssetSpec:
            spec = super().get_asset_spec(data)
            return spec.replace_attributes(
                key=spec.key.with_prefix("my_prefix"),
            )

    resource = SigmaOrganization(
        base_url=SigmaBaseUrl.AWS_US,
        client_id=EnvVar("SIGMA_CLIENT_ID"),
        client_secret=EnvVar("SIGMA_CLIENT_SECRET"),
    )

    # Pass the translator type
    sigma_specs = load_sigma_asset_specs(resource, dagster_sigma_translator=MyCoolTranslator)
    defs = Definitions(assets=[*sigma_specs], jobs=[define_asset_job("all_asset_job")])
